@@ -1,12 +1,12 @@
-from collections import defaultdict
-import os
-import json
 import argparse
+import json
+import os
+from collections import defaultdict
 
-from celescope.tools.utils import add_log
+import celescope.tools.utils as utils
 
 
-@add_log
+@utils.add_log
 def merge_report():
     parser = argparse.ArgumentParser('merge report')
     parser.add_argument('--outdir', help='outdir', required=True)
@@ -29,14 +29,7 @@ def merge_report():
         data_dic = json.load(open(data_json))
         for summary in summarys:
             if summary not in data_dic.keys():
-                # TCR and BCR data_dict has prefix
-                bool_in = False
-                for key in data_dic.keys():
-                    if key.find(summary) != -1:
-                        summary = key
-                        bool_in = True
-                if not bool_in:
-                    continue
+                continue
 
             # add title
             if sample == samples[0]:
@@ -58,7 +51,7 @@ def merge_report():
         rm_files()
 
 
-@add_log
+@utils.add_log
 def rm_files():
     cmd = '''
         find . -iname '*.fq*' -delete;
