@@ -49,7 +49,6 @@ class Multi():
 
         self.outdir_dic = {}
 
-
     def common_args(self):
         readme = f'{self.__ASSAY__} multi-samples'
         parser = argparse.ArgumentParser(readme,
@@ -70,6 +69,7 @@ Mapfile is a tab-delimited text file with as least three columns. Each line of m
 - `tag` Required, matched_dir.
 - `dynaseq` Optional, forced cell number.
 - `snp` Required, matched_dir.
+- `capture_virus` Required, matched_dir.
 
 5th column:
 - `dynaseq` Required, background snp file.
@@ -92,8 +92,10 @@ fastq_prefix2_1.fq.gz	fastq_prefix2_2.fq.gz
 ```
 ''',
             required=True)
-        parser.add_argument('--mod', help='Which type of script to generate, `sjm` or `shell`.', choices=['sjm', 'shell'], default='sjm')
-        parser.add_argument('--rm_files', action='store_true', help='Remove redundant fastq and bam files after running.')
+        parser.add_argument('--mod', help='Which type of script to generate, `sjm` or `shell`.',
+                            choices=['sjm', 'shell'], default='sjm')
+        parser.add_argument('--rm_files', action='store_true',
+                            help='Remove redundant fastq and bam files after running.')
         parser.add_argument('--steps_run', help='Steps to run. Multiple Steps are separated by comma.', default='all')
         # sub_program parser do not have
         parser.add_argument('--outdir', help='Output directory.', default="./")
@@ -363,7 +365,6 @@ job_end
             os.system('mkdir -p ./shell/')
             for sample in self.shell_dict:
                 with open(f'./shell/{sample}.sh', 'w') as f:
-                    f.write("set -eo pipefail\n")
                     f.write(self.shell_dict[sample])
 
     def run(self):
